##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Remote object traverser.

$Id: remoteproxy.py 115190 2010-07-29 06:52:35Z icemac $
"""
__docformat__ = "reStructuredText"
import zope.interface
from zope.interface import declarations
from zope.proxy import getProxiedObject, removeAllProxies
from zope.proxy.decorator import DecoratorSpecificationDescriptor
from zope.publisher.interfaces.browser import IBrowserPublisher, IBrowserRequest

from z3c.mountpoint import interfaces

class RemoteLocationProxyDecoratorSpecificationDescriptor(
    DecoratorSpecificationDescriptor):

    def __get__(self, inst, cls=None):
        if inst is None:
            return declarations.getObjectSpecification(cls)
        else:
            provided = zope.interface.providedBy(getProxiedObject(inst))

            # Use type rather than __class__ because inst is a proxy and
            # will return the proxied object's class.
            cls = type(inst)
            # Create a special version of Provides, which forces the remote
            # location proxy to the front, so that a special traverser can be
            # enforced.
            return declarations.Provides(
                cls, interfaces.IRemoteLocationProxy, provided)


class RemoteLocationProxy(zope.location.LocationProxy):
    """A location proxy for remote objects."""
    __providedBy__ = RemoteLocationProxyDecoratorSpecificationDescriptor()


class RemoteLocationProxyTraverser(object):
    zope.component.adapts(interfaces.IRemoteLocationProxy, IBrowserRequest)
    zope.interface.implements(IBrowserPublisher)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def browserDefault(self, request):
        ob = self.context
        view_name = zapi.getDefaultViewName(ob, request)
        return ob, (view_name,)

    def publishTraverse(self, request, name):
        pureContext = removeAllProxies(self.context)
        traverser = zope.component.getMultiAdapter(
            (pureContext, self.request), IBrowserPublisher)
        result = traverser.publishTraverse(request, name)
        # Only remove the security proxy from the context.
        return RemoteLocationProxy(result, getProxiedObject(self.context), name)



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""A simple multi-database mount-point implementation

$Id: mountpoint.py 115190 2010-07-29 06:52:35Z icemac $
"""
__docformat__ = "reStructuredText"
import persistent
import zope.component
import zope.interface
import zope.lifecycleevent
import zope.location
from zope.app.container import contained
from zope.app.publication import traversers
from zope.publisher.interfaces.browser import IBrowserRequest, IBrowserPublisher
from zope.security.proxy import removeSecurityProxy
from zope.traversing import api

from z3c.mountpoint import interfaces, remoteproxy

class MountPoint(persistent.Persistent, contained.Contained):
    """A simple mount point."""
    zope.interface.implements(interfaces.IMountPoint)

    def __init__(self, connectionName=None, objectPath=u'/', objectName=u'object'):
        self.connectionName = connectionName
        self.objectPath = objectPath
        self.objectName = objectName
        self.object = None

    def update(self):
        parent = self.__parent__
        if parent is not None:
            # Get the connection by name
            conn = parent._p_jar.get_connection(self.connectionName)
            obj = conn.root()['Application']
            obj = api.traverse(obj, self.objectPath)
            self.object = obj
        else:
            self.object = None

    @apply
    def __parent__():
        def get(self):
            return self and self.__dict__.get('__parent__', None)
        def set(self, parent):
            self.__dict__['__parent__'] = parent
            self.update()
        return property(get, set)


class MountPointTraverser(traversers.SimpleComponentTraverser):

    zope.component.adapts(interfaces.IMountPoint, IBrowserRequest)
    zope.interface.implementsOnly(IBrowserPublisher)

    def publishTraverse(self, request, name):
        if name == self.context.objectName:
            # Remove the security proxy, because we need a bare object to wrap
            # the location proxy around.
            context = removeSecurityProxy(self.context)
            return remoteproxy.RemoteLocationProxy(
                context.object, context, self.context.objectName)
        return super(MountPointTraverser, self).publishTraverse(
            request, name)


@zope.component.adapter(
    interfaces.IMountPoint, zope.lifecycleevent.IObjectModifiedEvent)
def updateMountedObject(mountPoint, event):
    mountPoint.update()



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""A simple multi-database mount-point implementation

$Id: interfaces.py 115190 2010-07-29 06:52:35Z icemac $
"""
__docformat__ = "reStructuredText"
import zope.interface
import zope.schema


class IMountPoint(zope.interface.Interface):
    """A simple mount point."""

    connectionName = zope.schema.Choice(
        title=u'Conenction Name',
        vocabulary='ZODB Connection Names',
        required=True)

    objectPath = zope.schema.TextLine(
        title=u'Object Path',
        default=u'/',
        required=True)

    objectName = zope.schema.TextLine(
        title=u'Object Name',
        description=u'The name under which the object will be known '
                    u'when traversing from the mount point.',
        default=u'object',
        required=True)

    object = zope.schema.Field(
        title=u'Foreign object')

    def update():
        """Update the mounted object."""


class IRemoteLocationProxy(zope.interface.Interface):
    """Remote Location Proxy

    When an object from a different object database is used, the directly
    specified parent is not the correct one anymore. The parent should point
    to the local mount point and subsequent objects in the path to the
    location proxied parent.
    """



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Browser code

$Id: browser.py 115190 2010-07-29 06:52:35Z icemac $
"""
__docformat__ = "reStructuredText"
from zope.app.publisher.browser import menu, menumeta
from zope.security.checker import CheckerPublic

from z3c.mountpoint import interfaces

def ObjectMenuItem(context, request):
    factory = menumeta.MenuItemFactory(
        menu.BrowserMenuItem,
        title=u'Object',
        action=context.objectName + '/@@SelectedManagementView.html',
        permission=CheckerPublic,
        order=2,
        _for=interfaces.IMountPoint)
    return factory(context, request)



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Browser code

$Id: tests.py 115190 2010-07-29 06:52:35Z icemac $
"""
__docformat__ = "reStructuredText"
from zope.app.publication import traversers
from zope.app.testing import placelesssetup, setup
from zope.publisher.interfaces.browser import IBrowserPublisher, IBrowserRequest
from zope.security import checker
from zope.site import folder
from zope.site.interfaces import IFolder
from zope.traversing.testing import setUp as traversalSetUp
import doctest
import unittest
import zope.component
import zope.interface

from z3c.mountpoint import interfaces, mountpoint

def setUp(test):
    placelesssetup.setUp(test)
    traversalSetUp()
    # The test traverser is all we need.
    zope.component.provideAdapter(
        traversers.TestTraverser,
        (zope.interface.Interface, IBrowserRequest), IBrowserPublisher)
    # A simple interface checker for: MountPoint
    checker.defineChecker(
        mountpoint.MountPoint, checker.InterfaceChecker(interfaces.IMountPoint))
    # A simple interface checker for: Folder
    checker.defineChecker(
        folder.Folder, checker.InterfaceChecker(IFolder))

def test_suite():
    return unittest.TestSuite((
        doctest.DocFileSuite('README.txt',
                     setUp=setUp, tearDown=placelesssetup.tearDown,
                     optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
                     ),
        ))



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""A simple multi-database mount-point implementation

$Id: connection.py 115190 2010-07-29 06:52:35Z icemac $
"""
__docformat__ = "reStructuredText"
from zope.schema import vocabulary
from zope.app.component import hooks

def ZODBConnectionNamesVocabulary(context):
    return vocabulary.SimpleVocabulary(
        [vocabulary.SimpleTerm(name)
         for name in hooks.getSite()._p_jar.db().databases])
